@@ -78,7 +78,11 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
     def test_equal_with_same_choices(self):
         t1 = types.String(choices=['foo', 'bar'])
         t2 = types.String(choices=['foo', 'bar'])
+        t3 = types.String(choices=('foo', 'bar'))
+        t4 = types.String(choices=['bar', 'foo'])
         self.assertTrue(t1 == t2)
+        self.assertTrue(t1 == t3)
+        self.assertTrue(t1 == t4)
 
     def test_not_equal_with_different_choices(self):
         t1 = types.String(choices=['foo', 'bar'])
@@ -126,6 +130,36 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
         t2 = types.String(regex=re.compile("^[a-z]"))
         self.assertFalse(t1 == t2)
 
+    def test_ignore_case(self):
+        self.type_instance = types.String(choices=['foo', 'bar'],
+                                          ignore_case=True)
+        self.assertConvertedValue('Foo', 'Foo')
+        self.assertConvertedValue('bAr', 'bAr')
+
+    def test_ignore_case_raises(self):
+        self.type_instance = types.String(choices=['foo', 'bar'],
+                                          ignore_case=False)
+        self.assertRaises(ValueError, self.assertConvertedValue, 'Foo', 'Foo')
+
+    def test_regex_and_ignore_case(self):
+        self.type_instance = types.String(regex=re.compile("^[A-Z]"),
+                                          ignore_case=True)
+        self.assertConvertedValue("foo", "foo")
+
+    def test_regex_and_ignore_case_str(self):
+        self.type_instance = types.String(regex="^[A-Z]", ignore_case=True)
+        self.assertConvertedValue("foo", "foo")
+
+    def test_regex_preserve_flags(self):
+        self.type_instance = types.String(regex=re.compile("^[A-Z]", re.I),
+                                          ignore_case=False)
+        self.assertConvertedValue("foo", "foo")
+
+    def test_max_length(self):
+        self.type_instance = types.String(max_length=5)
+        self.assertInvalid('123456')
+        self.assertConvertedValue('12345', '12345')
+
 
 class BooleanTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.Boolean()
@@ -212,6 +246,10 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.Integer(min=0, max=0)
         self.assertEqual('Integer(min=0, max=0)', repr(t))
 
+    def test_repr_with_choices(self):
+        t = types.Integer(choices=[80, 457])
+        self.assertEqual('Integer(choices=[80, 457])', repr(t))
+
     def test_equal(self):
         self.assertTrue(types.Integer() == types.Integer())
 
@@ -226,12 +264,35 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
         t2 = types.Integer(min=1, max=123)
         self.assertTrue(t1 == t2)
 
+    def test_equal_with_same_choices(self):
+        t1 = types.Integer(choices=[80, 457])
+        t2 = types.Integer(choices=[457, 80])
+        self.assertTrue(t1 == t2)
+
     def test_not_equal(self):
         self.assertFalse(types.Integer(min=123) == types.Integer(min=456))
+        self.assertFalse(types.Integer(choices=[80, 457]) ==
+                         types.Integer(choices=[80, 40]))
+        self.assertFalse(types.Integer(choices=[80, 457]) ==
+                         types.Integer())
 
     def test_not_equal_to_other_class(self):
         self.assertFalse(types.Integer() == types.String())
 
+    def test_choices_with_min_max(self):
+        self.assertRaises(ValueError,
+                          types.Integer,
+                          min=10,
+                          choices=[50, 60])
+        self.assertRaises(ValueError,
+                          types.Integer,
+                          max=100,
+                          choices=[50, 60])
+        self.assertRaises(ValueError,
+                          types.Integer,
+                          min=10, max=100,
+                          choices=[50, 60])
+
     def test_min_greater_max(self):
         self.assertRaises(ValueError,
                           types.Integer,
@@ -275,6 +336,22 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertRaises(ValueError, t, 201)
         self.assertRaises(ValueError, t, -457)
 
+    def test_with_choices_list(self):
+        t = types.Integer(choices=[80, 457])
+        self.assertRaises(ValueError, t, 1)
+        self.assertRaises(ValueError, t, 200)
+        self.assertRaises(ValueError, t, -457)
+        t(80)
+        t(457)
+
+    def test_with_choices_tuple(self):
+        t = types.Integer(choices=(80, 457))
+        self.assertRaises(ValueError, t, 1)
+        self.assertRaises(ValueError, t, 200)
+        self.assertRaises(ValueError, t, -457)
+        t(80)
+        t(457)
+
 
 class FloatTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.Float()
@@ -474,3 +551,71 @@ class IPv6AddressTypeTests(IPAddressTypeTests):
 
     def test_ipv4_address(self):
         self.assertInvalid('192.168.0.1')
+
+
+class HostnameTypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.Hostname()
+
+    def assertConvertedEqual(self, value):
+        self.assertConvertedValue(value, value)
+
+    def test_empty_hostname_fails(self):
+        self.assertInvalid('')
+
+    def test_should_return_same_hostname_if_valid(self):
+        self.assertConvertedEqual('foo.bar')
+
+    def test_trailing_quote_is_invalid(self):
+        self.assertInvalid('foo.bar"')
+
+    def test_repr(self):
+        self.assertEqual('Hostname', repr(types.Hostname()))
+
+    def test_equal(self):
+        self.assertEqual(types.Hostname(), types.Hostname())
+        self.assertEqual(types.Hostname(), types.Hostname())
+
+    def test_not_equal_to_other_class(self):
+        self.assertNotEqual(types.Hostname(), types.Integer())
+        self.assertNotEqual(types.Hostname(), types.String())
+
+    def test_invalid_characters(self):
+        self.assertInvalid('"host"')
+        self.assertInvalid("h'ost'")
+        self.assertInvalid("h'ost")
+        self.assertInvalid("h$ost")
+        self.assertInvalid("h%ost")
+        self.assertInvalid("host_01.co.uk")
+        self.assertInvalid("host;name=99")
+        self.assertInvalid('___site0.1001')
+        self.assertInvalid('_site01001')
+        self.assertInvalid("host..name")
+        self.assertInvalid(".host.name.com")
+        self.assertInvalid("no spaces")
+
+    def test_no_start_end_hyphens(self):
+        self.assertInvalid("-host.com")
+        self.assertInvalid("-hostname.com-")
+        self.assertInvalid("hostname.co.uk-")
+
+    def test_strip_trailing_dot(self):
+        self.assertConvertedValue('cell1.nova.site1.', 'cell1.nova.site1')
+        self.assertConvertedValue('cell1.', 'cell1')
+
+    def test_valid_hostname(self):
+        self.assertConvertedEqual('cell1.nova.site1')
+        self.assertConvertedEqual('site01001')
+        self.assertConvertedEqual('home-site-here.org.com')
+        self.assertConvertedEqual('192.168.0.1')
+        self.assertConvertedEqual('1.1.1')
+        self.assertConvertedEqual('localhost')
+
+    def test_max_segment_size(self):
+        self.assertConvertedEqual('host.%s.com' % ('x' * 63))
+        self.assertInvalid('host.%s.com' % ('x' * 64))
+
+    def test_max_hostname_size(self):
+        test_str = '.'.join('x'*31 for x in range(8))
+        self.assertEqual(255, len(test_str))
+        self.assertInvalid(test_str)
+        self.assertConvertedEqual(test_str[:-2])
